////////////////////////////////////////////////////////////////////////////////
// This file is part of the exemplary solution of Lab3 of the course
// "Software Engineering using Formal Methods" (2005)
// and playground for the verification tasks of Lab4
////////////////////////////////////////////////////////////////////////////////

/**
 * Singleton class for representing an assumed system-wide clock (for simplicity
 * we suppose that both the central host and all ATMs are running on the same
 * clock).
 * @stereotype singleton
 */
public class Clock {
    /**
     * @supplierRole instance
     * @supplierCardinality 1 
     */
    private static final Clock instance = new Clock();
    
    /**
     * The current date/time. We don't care about details of this representation
     * ...
     */
    private int currentDate = 0;
    
    /**
     * Private constructor as part of the applied singleton design pattern
     */
    private Clock() {}
    
    /**
     * @return Returns the instance.
     */
    public static Clock getInstance () {
        return instance;
    }

    /**
     * Make the time progress
     */
    public static void tick () {
        getInstance ().currentDate++;
    }
    
    /**
     * @return the current date/time
     * 
     * @stereotype query
     */
    public static int getCurrentDate () {
        return getInstance ().currentDate;
    }
    
    /**
     * @return some date/time in a very distant past (used to initialise certain
     *         attributes in other classes)
     * 
     * @stereotype query
     */
    public static int getBigBangsDate() {
        return -1000;
    }

    /**
     * @return <code>true</code> iff the two given dates refer to the same day
     * 
     * @stereotype query
     */
    public static boolean isSameDay (int dateA, int dateB) {
        // dummy implementation
        return isSameInterval ( dateA, dateB, 10 );
    }

    /**
     * @return <code>true</code> iff <code>dateA</code> is an earlier date
     *         than <code>dateB</code>
     * 
     * @stereotype query
     */
    public static boolean isEarlier (int dateA, int dateB) {
        return dateA < dateB;
    }
    
    /**
     * @return <code>true</code> iff the two given dates refer to the same day
     * 
     * @stereotype query
     * @postconditions isSameDay(dateA, dateB) implies isSameMonth(dateA, dateB)
     */
    public static boolean isSameMonth (int dateA, int dateB) {
        // dummy implementation
        return isSameInterval ( dateA, dateB, 100 );
    }

    /**
     * @stereotype query
     */
    private static boolean isSameInterval (int dateA,
                                           int dateB,
                                           int intervalLength) {
        return dateA / intervalLength == dateB / intervalLength;
    }
}
