////////////////////////////////////////////////////////////////////////////////
// This file is part of the exemplary solution of Lab3 of the course
// "Software Engineering using Formal Methods" (2005)
// and playground for the verification tasks of Lab4
////////////////////////////////////////////////////////////////////////////////

/**
 * Class for representing the transaction kind withdrawal, i.e. the act of
 * withdrawing a certain amount of money from an account.
 * Objects of this class are immutable
 * 
 * @invariants amount > 0
 */
public class Withdrawal extends Transaction {
    
    /**
     * The amount of money that is withdrawn
     */
    private final int amount;

    /**
     * TODO: I guess that using KeY it is currently not possible to verify the
     * invariant (I think the bugtracker contains an entry for that)
     * 
     * @preconditions amount > 0
     */
    public Withdrawal (int date, int amount) {
        super ( date );
        this.amount = amount;        
    }

    /**
     * The design pattern "Strategy" is used for implementing the
     * synchronisation of offline account proxies with the permanent accounts.
     * Invoking this method carries out <code>this</code> transaction on the
     * real account. Note that the withdrawal from the permanent account is
     * performed for the correct date
     * 
     * @param target
     *            the permanent account on which <code>this</code> transaction
     *            is supposed to be carried out
     */
    public void replay (PermanentAccount target) {
        target.withdraw ( getDate (), getAmount () );
    }

    /**
     * @return the amount of money that is withdrawn
     * 
     * @stereotype query
     */
    public int getAmount () {
        return amount;
    }

    
    public String toString () {
        return "" + getDate() + ":Withdrawal: " + getAmount();
    }
}
